// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/driver/ioctl.c $
// $Rev: 52660 $
// $Date: 2023-03-21 10:16:50 -0500 (Tue, 21 Mar 2023) $

// 24DSI64C200K: Device Driver: source file

#include "main.h"



// macros *********************************************************************

#define	AUTO_CAL_START			D7		// BCTLR
#define	AUTO_CAL_PASS			D12		// BCTLR
#define	INIT_START				D15		// BCTLR

#define	IRQ_INIT_DONE			0x0000	// BCTLR
#define	IRQ_AUTO_CAL_DONE		0x0100	// BCTLR
#define	IRQ_MASK				0x0700	// BCTLR
#define	IRQ_ACTIVE				0x0800	// BCTLR



//*****************************************************************************
static int _query(dev_data_t* dev, s32* arg)
{
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			arg[0]	= DSI64C200K_IOCTL_QUERY_ERROR;
			ret		= -EINVAL;
			break;

		case DSI64C200K_QUERY_AUTO_CAL_MS:	arg[0]	= dev->cache.auto_cal_ms;	break;
		case DSI64C200K_QUERY_CHANNEL_MAX:	arg[0]	= dev->cache.channels_max;	break;
		case DSI64C200K_QUERY_CHANNEL_QTY:	arg[0]	= dev->cache.channel_qty;	break;
		case DSI64C200K_QUERY_COUNT:		arg[0]	= DSI64C200K_QUERY_LAST;	break;
		case DSI64C200K_QUERY_DEVICE_TYPE:	arg[0]	= dev->board_type;			break;
		case DSI64C200K_QUERY_FGEN_MAX:		arg[0]	= dev->cache.fgen_max;		break;
		case DSI64C200K_QUERY_FGEN_MIN:		arg[0]	= dev->cache.fgen_min;		break;
		case DSI64C200K_QUERY_FIFO_SIZE:	arg[0]	= dev->cache.fifo_size;		break;
		case DSI64C200K_QUERY_FILTER_FREQ:	arg[0]	= dev->cache.filter_freq;	break;
		case DSI64C200K_QUERY_FREF_DEFAULT:	arg[0]	= dev->cache.fref_default;	break;
		case DSI64C200K_QUERY_FSAMP_MAX:	arg[0]	= dev->cache.fsamp_max;		break;
		case DSI64C200K_QUERY_FSAMP_MIN:	arg[0]	= dev->cache.fsamp_min;		break;
		case DSI64C200K_QUERY_FW_REV:		arg[0]	= dev->cache.fw_rev;		break;
		case DSI64C200K_QUERY_INIT_ADC_MS:	arg[0]	= dev->cache.init_adc_ms;	break;
		case DSI64C200K_QUERY_INIT_MS:		arg[0]	= dev->cache.initialize_ms;	break;
		case DSI64C200K_QUERY_MASTER_CLOCK:	arg[0]	= dev->cache.master_clock;	break;
		case DSI64C200K_QUERY_MCLK_MAX:		arg[0]	= dev->cache.mclk_max;		break;
		case DSI64C200K_QUERY_MCLK_MIN:		arg[0]	= dev->cache.mclk_min;		break;
		case DSI64C200K_QUERY_NREF_MAX:		arg[0]	= dev->cache.nref_max;		break;
		case DSI64C200K_QUERY_NREF_MIN:		arg[0]	= dev->cache.nref_min;		break;
		case DSI64C200K_QUERY_NREF_OPT_MAX:	arg[0]	= dev->cache.nref_opt_max;	break;
		case DSI64C200K_QUERY_NREF_OPT_MIN:	arg[0]	= dev->cache.nref_opt_min;	break;
		case DSI64C200K_QUERY_NVCO_MAX:		arg[0]	= dev->cache.nvco_max;		break;
		case DSI64C200K_QUERY_NVCO_MIN:		arg[0]	= dev->cache.nvco_min;		break;
		case DSI64C200K_QUERY_NVCO_OPT_MAX:	arg[0]	= dev->cache.nref_opt_max;	break;
		case DSI64C200K_QUERY_NVCO_OPT_MIN:	arg[0]	= dev->cache.nref_opt_min;	break;
		case DSI64C200K_QUERY_V_RANGE:		arg[0]	= dev->cache.voltage_range;	break;
	}

	return(ret);
}



//*****************************************************************************
static int _init_start(dev_data_t* dev, void* arg)
{
	// Initiate initialization.
	os_reg_mem_tx_u32(NULL, dev->vaddr.gsc_bctlr_32, INIT_START);
	return(0);
}



//*****************************************************************************
int initialize_ioctl(dev_data_t* dev, void* arg)
{
	u32			bctlr;
	int			i;
	int			mask;
	long		ms			= dev->cache.initialize_ms + 5000;
	long		ms_total	= ms;
	os_sem_t	sem;
	int			ret			= 0;
	int			tmp;
	int			value;
	gsc_wait_t	wt;

	if (dev->irq.opened)
	{
		ms_total	*= 2;

		// Safely select the Initialize Done interrupt.
		mask	= IRQ_MASK | IRQ_ACTIVE;
		value	= IRQ_INIT_DONE;
		os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bctlr_32, value, mask);

		// Wait for the local interrupt.
		os_sem_create(&sem);	// dummy, required for wait operations.
		memset(&wt, 0, sizeof(wt));
		wt.flags		= GSC_WAIT_FLAG_INTERNAL;
		wt.gsc			= DSI64C200K_WAIT_GSC_INIT_DONE;
		wt.timeout_ms	= ms;
		ret				= gsc_wait_event(dev, &wt, _init_start, NULL, &sem);
		os_sem_destroy(&sem);

		if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
		{
			ret	= ret ? ret : -ETIMEDOUT;
			printf(	"%s: INITILIZE DONE IRQ TIMED OUT AFTER %ld ms.\n",
					dev->model,
					(long) ms);
		}
	}

	// Manually wait for completion as the IRQ wait may have ended early.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bctlr_32, INIT_START, 0);

	if (tmp)
	{
		ret	= ret ? ret : tmp;
		printf(	"%s: INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	bctlr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bctlr_32);

	if (bctlr & INIT_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: INITILIZE STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms_total);
	}

	// Initialize the software settings.

	for (i = 0; i < DEV_IO_STREAM_QTY; i++)
	{
		if (dev->io.io_streams[i])
		{
			if (dev->io.io_streams[i]->dev_io_sw_init)
			{
				(dev->io.io_streams[i]->dev_io_sw_init)(dev, dev->io.io_streams[i]);
			}
		}
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_start(dev_data_t* dev, void* arg)
{
	// Initiate auto-calibration.
	os_reg_mem_mx_u32(NULL, dev->vaddr.gsc_bctlr_32, AUTO_CAL_START, AUTO_CAL_START);
	return(0);
}



//*****************************************************************************
static int _auto_calibrate(dev_data_t* dev, void* arg)
{
	u32			bctlr;
	u32			mask;
	long		ms		= dev->cache.auto_cal_ms + 5000;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			tmp;
	u32			value;
	gsc_wait_t	wt;

	// Safely select the Auto-Cal Done interrupt.
	mask	= IRQ_MASK | IRQ_ACTIVE;
	value	= IRQ_AUTO_CAL_DONE;
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bctlr_32, value, mask);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= DSI64C200K_WAIT_GSC_AUTO_CAL_DONE;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(dev, &wt, _auto_cal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATE IRQ TIMED OUT AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually wait for completion in case something terminates our wait early.
	tmp	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bctlr_32, AUTO_CAL_START, 0);

	if (tmp)
	{
		ms	*= 2;
		ret	= ret ? ret : tmp;
		printf(	"%s: AUTOCALIBRATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				ms);
	}

	// Manually check for completion as the IRQ wait may have ended early.
	bctlr	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bctlr_32);

	if (bctlr & AUTO_CAL_START)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTOCALIBRATION STILL ACTIVE AFTER %ld ms.\n",
				dev->model,
				ms);
	}

	// Final results.
	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bctlr_32);

	if ((reg & AUTO_CAL_PASS) == 0)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTOCALIBRATION FAILED (%ld ms).\n",
				dev->model,
				ms);
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_sts(dev_data_t* dev, s32* arg)
{
	u32	reg;

	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bctlr_32);

	if (reg & AUTO_CAL_START)
		arg[0]	= DSI64C200K_AUTO_CAL_STS_ACTIVE;
	else if (reg & AUTO_CAL_PASS)
		arg[0]	= DSI64C200K_AUTO_CAL_STS_PASS;
	else
		arg[0]	= DSI64C200K_AUTO_CAL_STS_FAIL;

	return(0);
}



//*****************************************************************************
static int _ai_buf_clear(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bufcr_32, D20, D20);
	return(0);
}



//*****************************************************************************
static int _ai_buf_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AI_BUF_ENABLE_NO,
		DSI64C200K_AI_BUF_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bufcr_32, 19, 19);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_fill_lvl(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0x40000, dev->vaddr.gsc_bufsr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_overflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AI_BUF_OVERFLOW_NO,
		DSI64C200K_AI_BUF_OVERFLOW_YES,
		DSI64C200K_AI_BUF_OVERFLOW_CLEAR,
		DSI64C200K_AI_BUF_OVERFLOW_TEST,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bufcr_32, 24, 24);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thr_sts(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AI_BUF_THR_STS_IDLE,
		DSI64C200K_AI_BUF_THR_STS_ACTIVE,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 14, 14);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_thresh(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0x7FFFF, dev->vaddr.gsc_bufcr_32, 18, 0);
	return(ret);
}



//*****************************************************************************
static int _ai_buf_underflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AI_BUF_UNDERFLOW_NO,
		DSI64C200K_AI_BUF_UNDERFLOW_YES,
		DSI64C200K_AI_BUF_UNDERFLOW_CLEAR,
		DSI64C200K_AI_BUF_UNDERFLOW_TEST,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bufcr_32, 25, 25);
	return(ret);
}



//*****************************************************************************
static int _ai_filter(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AI_FILTER_LOW_LATENCY,
		DSI64C200K_AI_FILTER_WIDEBAND,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_aicr_32, 23, 23);
	return(ret);
}



//*****************************************************************************
static int _ai_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AI_MODE_DIFF,
		DSI64C200K_AI_MODE_ZERO,
		DSI64C200K_AI_MODE_VREF,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 1, 0);
	return(ret);
}



//*****************************************************************************
static int _aux_clock(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AUX_CLOCK_INACTIVE,
		DSI64C200K_AUX_CLOCK_INPUT,
		DSI64C200K_AUX_CLOCK_OUTPUT,
		-1	// terminate list
	};

	int	ret;
	s32	v	= arg[0];

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_asiocr_32, 1, 0);

	if ((v == -1) && (arg[0] == 3))
		arg[0]	= DSI64C200K_AUX_CLOCK_OUTPUT;

	return(ret);
}



//*****************************************************************************
static int _aux_sync(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_AUX_SYNC_INACTIVE,
		DSI64C200K_AUX_SYNC_INPUT,
		DSI64C200K_AUX_SYNC_OUTPUT,
		-1	// terminate list
	};

	int	ret;
	s32	v	= arg[0];

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_asiocr_32, 3, 2);

	if ((v == -1) && (arg[0] == 3))
		arg[0]	= DSI64C200K_AUX_SYNC_OUTPUT;

	return(ret);
}



//*****************************************************************************
static int _burst_enable(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_BURST_ENABLE_NO,
		DSI64C200K_BURST_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 22, 22);
	return(ret);
}



//*****************************************************************************
static int _burst_rate_div(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_bttr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _burst_size(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFFFFFF, dev->vaddr.gsc_bbsr_32, 23, 0);
	return(ret);
}



//*****************************************************************************
static int _burst_timer(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_BURST_TIMER_DISABLE,
		DSI64C200K_BURST_TIMER_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 16, 16);
	return(ret);
}



//*****************************************************************************
static int _burst_trigger(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bctlr_32, D23, D23);
	return(0);
}



//*****************************************************************************
static int _chan_grp_active(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xFF, dev->vaddr.gsc_aicr_32, 7, 0);
	return(ret);
}



//*****************************************************************************
static int _channels_ready(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_CHANNELS_READY_NO,
		DSI64C200K_CHANNELS_READY_YES,
		DSI64C200K_CHANNELS_READY_WAIT,
		DSI64C200K_CHANNELS_READY_TEST,
		-1	// terminate list
	};

	int	ret	= 0;

	if (arg[0] == DSI64C200K_CHANNELS_READY_WAIT)
	{
		gsc_poll_u32(dev, 1000, dev->vaddr.gsc_bctlr_32, D13, D13);
		arg[0]	= DSI64C200K_CHANNELS_READY_TEST;
	}

	if (arg[0] == DSI64C200K_CHANNELS_READY_TEST)
		ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 13, 13);
	else
		ret	= -EINVAL;

	return(ret);
}



//*****************************************************************************
static int _clk_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_CLK_SRC_RATE_GEN,
		DSI64C200K_CLK_SRC_EXT_FGEN,
		DSI64C200K_CLK_SRC_EXT_MCLK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_aicr_32, 21, 20);
	return(ret);
}



//*****************************************************************************
static int _control_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_CONTROL_MODE_TARGET,
		DSI64C200K_CONTROL_MODE_INITIATOR,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _data_format(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_DATA_FORMAT_2S_COMP,
		DSI64C200K_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _data_width(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_DATA_WIDTH_16,
		DSI64C200K_DATA_WIDTH_18,
		DSI64C200K_DATA_WIDTH_20,
		DSI64C200K_DATA_WIDTH_24,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bufcr_32, 22, 21);
	return(ret);
}



//*****************************************************************************
static int _dio_dir_out(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_diopr_32, 11, 8);
	return(ret);
}



//*****************************************************************************
static int _dio_read(dev_data_t* dev, s32* arg)
{
	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_diopr_32, 3, 0);
	return(ret);
}



//*****************************************************************************
static int _dio_write(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev, arg, 0, 0xF, dev->vaddr.gsc_diopr_32, 3, 0);
	return(ret);
}



//*****************************************************************************
static int _ext_clk_src(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_EXT_CLK_SRC_MCLK,
		DSI64C200K_EXT_CLK_SRC_GEN,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 18, 18);
	return(ret);
}



//*****************************************************************************
static int _fgen_div(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_FGEN_DIV_1,
		DSI64C200K_FGEN_DIV_2,
		DSI64C200K_FGEN_DIV_4,
		DSI64C200K_FGEN_DIV_8,
		DSI64C200K_FGEN_DIV_16,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_aicr_32, 11, 8);
	return(ret);
}



//*****************************************************************************
static int _init_adcs(dev_data_t* dev, void* arg)
{
	long	ms;
	u32		reg;
	int		ret;

	// Manually wait for completion to insure that initialization completes
	// even if we get a signal, such as ^c.
	os_reg_mem_tx_u32(dev, dev->vaddr.gsc_bctlr_32, D19);
	ms	= dev->cache.init_adc_ms + 5000;
	ret	= gsc_poll_u32(dev, ms, dev->vaddr.gsc_bctlr_32, D19, 0);
	reg	= os_reg_mem_rx_u32(dev, dev->vaddr.gsc_bctlr_32);

	if (ret)
	{
		printf(	"%s: ADC INITIALIZATION TIMED OUT AFTER %ld ms.\n",
				dev->model,
				(long) ms);
	}

	if (reg & D19)
	{
		ret	= -EIO;
		printf(	"%s: ADC INITIALIZATION DID NOT COMPLETE WITHIN %ld ms.\n",
				dev->model,
				(long) ms);
	}

	return(ret);
}



//*****************************************************************************
static int _irq_sel(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_IRQ_INIT_DONE,
		DSI64C200K_IRQ_AUTO_CAL_DONE,
		DSI64C200K_IRQ_CHAN_READY_L2H,
		DSI64C200K_IRQ_AI_BUF_THR_L2H,
		DSI64C200K_IRQ_AI_BUF_THR_H2L,
		DSI64C200K_IRQ_AI_BURST_DONE,
		DSI64C200K_IRQ_CHAN_READY_H2L,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 10, 8);
	return(ret);
}



//*****************************************************************************
static int _mclk_div(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_MCLK_DIV_32_ECO,
		DSI64C200K_MCLK_DIV_8_MEDIAN,
		DSI64C200K_MCLK_DIV_4_FAST,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_aicr_32, 15, 12);
	return(ret);
}



//*****************************************************************************
static int _nref(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev,
							arg,
							dev->cache.nref_min,
							dev->cache.nref_max,
							dev->vaddr.gsc_rcr_32,
							27, 16);
	return(ret);
}



//*****************************************************************************
static int _nvco(dev_data_t* dev, void* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(dev,
							arg,
							dev->cache.nvco_min,
							dev->cache.nvco_max,
							dev->vaddr.gsc_rcr_32,
							11, 0);
	return(ret);
}



//*****************************************************************************
static int _over_sample(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_OVER_SAMPLE_32,
		DSI64C200K_OVER_SAMPLE_64,
		DSI64C200K_OVER_SAMPLE_128,
		DSI64C200K_OVER_SAMPLE_1024,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_aicr_32, 19, 16);
	return(ret);
}



//*****************************************************************************
static int _rx_io_abort(dev_data_t* dev, s32* arg)
{
	arg[0]	= gsc_read_abort_active_xfer(dev, &dev->io.rx);
	return(0);
}



//*****************************************************************************
static int _rx_io_mode(dev_data_t* dev, s32* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_BMDMA,
		GSC_IO_MODE_DMDMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &dev->io.rx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _rx_io_overflow(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_IO_OVERFLOW_IGNORE,
		DSI64C200K_IO_OVERFLOW_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.overflow_check);
	return(ret);
}



//*****************************************************************************
static int _rx_io_timeout(dev_data_t* dev, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			DSI64C200K_IO_TIMEOUT_MIN,
			DSI64C200K_IO_TIMEOUT_INFINITE,
			&dev->io.rx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _rx_io_underflow(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_IO_UNDERFLOW_IGNORE,
		DSI64C200K_IO_UNDERFLOW_CHECK,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, options, &dev->io.rx.underflow_check);
	return(ret);
}



//*****************************************************************************
static int _sw_sync(dev_data_t* dev, void* arg)
{
	os_reg_mem_mx_u32(dev, dev->vaddr.gsc_bctlr_32, D6, D6);
	return(0);
}



//*****************************************************************************
static int _sw_sync_mode(dev_data_t* dev, void* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_SW_SYNC_MODE_SYNC,
		DSI64C200K_SW_SYNC_MODE_CLR_BUF,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 17, 17);
	return(ret);
}



//*****************************************************************************
static int _sync_src(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_SYNC_SRC_TIMER_BCR,
		DSI64C200K_SYNC_SRC_EXT_BCR,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_aicr_32, 22, 22);
	return(ret);
}



//*****************************************************************************
static int _xcvr_type(dev_data_t* dev, s32* arg)
{
	static const s32	options[]	=
	{
		DSI64C200K_XCVR_TYPE_LVDS,
		DSI64C200K_XCVR_TYPE_TTL,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(dev, arg, options, dev->vaddr.gsc_bctlr_32, 21, 21);
	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ DSI64C200K_IOCTL_REG_READ,			(void*) gsc_reg_read_ioctl		},
	{ DSI64C200K_IOCTL_REG_WRITE,			(void*) gsc_reg_write_ioctl		},
	{ DSI64C200K_IOCTL_REG_MOD,				(void*) gsc_reg_mod_ioctl		},
	{ DSI64C200K_IOCTL_QUERY,				(void*) _query 					},
	{ DSI64C200K_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ DSI64C200K_IOCTL_AUTO_CALIBRATE,		(void*) _auto_calibrate			},
	{ DSI64C200K_IOCTL_AUTO_CAL_STS,		(void*) _auto_cal_sts 			},
	{ DSI64C200K_IOCTL_AI_BUF_CLEAR,		(void*) _ai_buf_clear 			},
	{ DSI64C200K_IOCTL_AI_BUF_ENABLE,		(void*) _ai_buf_enable 			},
	{ DSI64C200K_IOCTL_AI_BUF_FILL_LVL,		(void*) _ai_buf_fill_lvl 		},
	{ DSI64C200K_IOCTL_AI_BUF_OVERFLOW,		(void*) _ai_buf_overflow 		},
	{ DSI64C200K_IOCTL_AI_BUF_THR_STS,		(void*) _ai_buf_thr_sts 		},
	{ DSI64C200K_IOCTL_AI_BUF_THRESH,		(void*) _ai_buf_thresh 			},
	{ DSI64C200K_IOCTL_AI_BUF_UNDERFLOW,	(void*) _ai_buf_underflow 		},
	{ DSI64C200K_IOCTL_AI_FILTER,			(void*) _ai_filter 				},
	{ DSI64C200K_IOCTL_AI_MODE,				(void*) _ai_mode 				},
	{ DSI64C200K_IOCTL_AUX_CLOCK,			(void*) _aux_clock 				},
	{ DSI64C200K_IOCTL_AUX_SYNC,			(void*) _aux_sync 				},
	{ DSI64C200K_IOCTL_BURST_ENABLE,		(void*) _burst_enable 			},
	{ DSI64C200K_IOCTL_BURST_RATE_DIV,		(void*) _burst_rate_div 		},
	{ DSI64C200K_IOCTL_BURST_SIZE,			(void*) _burst_size 			},
	{ DSI64C200K_IOCTL_BURST_TIMER,			(void*) _burst_timer 			},
	{ DSI64C200K_IOCTL_BURST_TRIGGER,		(void*) _burst_trigger 			},
	{ DSI64C200K_IOCTL_CHAN_GRP_ACTIVE,		(void*) _chan_grp_active 		},
	{ DSI64C200K_IOCTL_CHANNELS_READY,		(void*) _channels_ready 		},
	{ DSI64C200K_IOCTL_CLK_SRC,				(void*) _clk_src 				},
	{ DSI64C200K_IOCTL_CONTROL_MODE,		(void*) _control_mode 			},
	{ DSI64C200K_IOCTL_DATA_FORMAT,			(void*) _data_format 			},
	{ DSI64C200K_IOCTL_DATA_WIDTH,			(void*) _data_width 			},
	{ DSI64C200K_IOCTL_DIO_DIR_OUT,			(void*) _dio_dir_out 			},
	{ DSI64C200K_IOCTL_DIO_READ,			(void*) _dio_read 				},
	{ DSI64C200K_IOCTL_DIO_WRITE,			(void*) _dio_write 				},
	{ DSI64C200K_IOCTL_EXT_CLK_SRC,			(void*) _ext_clk_src 			},
	{ DSI64C200K_IOCTL_FGEN_DIV,			(void*) _fgen_div 				},
	{ DSI64C200K_IOCTL_INIT_ADCS,			(void*) _init_adcs 				},
	{ DSI64C200K_IOCTL_IRQ_SEL,				(void*) _irq_sel 				},
	{ DSI64C200K_IOCTL_MCLK_DIV,			(void*) _mclk_div 				},
	{ DSI64C200K_IOCTL_NREF,				(void*) _nref 					},
	{ DSI64C200K_IOCTL_NVCO,				(void*) _nvco 					},
	{ DSI64C200K_IOCTL_OVER_SAMPLE,			(void*) _over_sample 			},
	{ DSI64C200K_IOCTL_RX_IO_ABORT,			(void*) _rx_io_abort 			},
	{ DSI64C200K_IOCTL_RX_IO_MODE,			(void*) _rx_io_mode 			},
	{ DSI64C200K_IOCTL_RX_IO_OVERFLOW,		(void*) _rx_io_overflow 		},
	{ DSI64C200K_IOCTL_RX_IO_TIMEOUT,		(void*) _rx_io_timeout 			},
	{ DSI64C200K_IOCTL_RX_IO_UNDERFLOW,		(void*) _rx_io_underflow 		},
	{ DSI64C200K_IOCTL_SW_SYNC,				(void*) _sw_sync 				},
	{ DSI64C200K_IOCTL_SW_SYNC_MODE,		(void*) _sw_sync_mode 			},
	{ DSI64C200K_IOCTL_SYNC_SRC,			(void*) _sync_src 				},
	{ DSI64C200K_IOCTL_WAIT_CANCEL,			(void*) gsc_wait_cancel_ioctl	},
	{ DSI64C200K_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ DSI64C200K_IOCTL_WAIT_STATUS,			(void*) gsc_wait_status_ioctl	},
	{ DSI64C200K_IOCTL_XCVR_TYPE,			(void*) _xcvr_type 				},
	{ -1, NULL }
};


