// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/driver/main.h $
// $Rev: 52660 $
// $Date: 2023-03-21 10:16:50 -0500 (Tue, 21 Mar 2023) $

// 24DSI64C200K: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "24dsi64c200k.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"24DSI64C200K"	// Upper case form of the below.
#define	DEV_NAME							"24dsi64c200k"	// MUST AGREE WITH DSI64C200K_BASE_NAME

#define	DEV_VERSION							"1.5"			// FOR DEVICE SPECIFIC CODE ONLY!
// 1.5	Updated to support the 6.x series kernel.
//		Updated white space for consistency.
// 1.4	Updated comments for consistency.
//		LINTed the source code.
//		Updated the Initialize and Autocal IOCTL services for consistency.
//		Updated macro use for improved maintainability.
//		Updated per PLX code segregation.
//		Updated start script.
// 1.3	Removed compiler warnings for Fedora 31.
//		Updated for porting to Windows.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		Updated some error messages.
// 1.2	Modified code so open succeeds even if initialization fails.
//		Updated ..._DMA macro spelling to ..._BMDMA.
//		Added support for the 5.x kernel series.
//		Added Endianness support.
// 1.1	Implemented metrics support.
//		Adjusted the lower sample rate limit.
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					DSI64C200K_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					DSI64C200K_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						DSI64C200K_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, DMA, DMDMA
	s32					overflow_check;		// Check overflow when reading?
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check underflow when reading?

	// Initialized by I/O service at time of use.

	int					abort;
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					board_index;	// Index of the device being accessed.
	int					users;			// Number of currently active open() requests.
	int					share;			// Were we opened in shared mode?

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrut support.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;			// Analog Input read
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bctlr_32;	// 0x00 Board Control Register
		VADDR_T			gsc_rcr_32;		// 0x04 Rate Control Register
		VADDR_T			gsc_diopr_32;	// 0x08 Digital I/O Port Register
		VADDR_T			gsc_aicr_32;	// 0x0C Analog Input Configuration Register
										// 0x10 Reserved
										// 0x14 Reserved
		VADDR_T			gsc_bbsr_32;	// 0x18 Burst Block Size Register
		VADDR_T			gsc_bttr_32;	// 0x1C Burst Trigger Timer Register
		VADDR_T			gsc_bufcr_32;	// 0x20 Buffer Control Register
		VADDR_T			gsc_bcfgr_32;	// 0x24 Board Configuration Register
		VADDR_T			gsc_bufsr_32;	// 0x28 Buffer Size Register
		VADDR_T			gsc_avr_32;		// 0x2C Auto Cal Values Register
		VADDR_T			gsc_idbr_32;	// 0x30 Input Data Buffer Register
		VADDR_T			gsc_asiocr_32;	// 0x34 Auxiliary SYNC I/O Control Register

	} vaddr;

	struct
	{
		u32				gsc_bcfgr_32;	// Board Configuration Register

		s32				auto_cal_ms;	// Maximum ms for auto-cal

		s32				channels_max;	// Maximum channels supported by model.
		s32				channel_qty;	// The number of channels on the device.

		s32				fgen_max;		// Rate Generator maximum output rate.
		s32				fgen_min;		// Rate Generator minimum output rate.
		s32				fifo_size;		// Size of FIFO - not the fill level.
		s32				filter_freq;	// -1, 900,000, ...
		s32				fref_default;	// 32,768,000
		s32				fsamp_max;		// The maximum Fsamp rate per channel.
		s32				fsamp_min;		// The minimum Fsamp rate per channel.
		s32				fw_rev;			// firmware revision

		s32				init_adc_ms;	// Maximum ms for ADC initialize
		s32				initialize_ms;	// Maximum ms for initialize

		s32				master_clock;	// The frequency of the master clock.
		s32				mclk_max;		// Mclk maximum rate.
		s32				mclk_min;		// Mclk minimum rate.

		s32				nref_max;		// Nref maximum
		s32				nref_min;		// Nref minimum
		s32				nref_opt_max;	// Nref maximum optimum
		s32				nref_opt_min;	// Nref minimum optimum
		s32				nvco_max;		// Nvco maximum
		s32				nvco_min;		// Nvco minimum
		s32				nvco_opt_max;	// Nvco maximum optimum
		s32				nvco_opt_min;	// Nvco minimum optimum

		s32				voltage_range;	// 10/...

	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t count);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
