// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/samples/dout/perform.c $
// $Rev: 52119 $
// $Date: 2022-12-01 11:44:17 -0600 (Thu, 01 Dec 2022) $

// 24DSI64C200K: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= dsi64c200k_initialize(args->fd, -1, 1);
	errs	+= dsi64c200k_dio_dir_out(args->fd, -1, 1, 0xF, NULL);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("    Seq   Port\n");
	printf("    No.   3210  Time Stamp      Delta\n");
	printf("    ====  ====  ==============  ==============\n");
}



//*****************************************************************************
static int _dev_dio_out(const args_t* args, u32 value)
{
	static	int				count	= 0;
	static	os_time_ns_t	t0;
	static	os_time_ns_t	ts;
	static	os_time_ns_t	ts_last;

	u32						bit;
	long long				delta;
	int						errs;
	int						i;

	if (count == 0)
	{
		os_time_get_ns(&t0);
		ts		= t0;
		ts_last	= t0;
	}

	ts_last	= ts;
	os_time_get_ns(&ts);
	errs	= dsi64c200k_dio_write(args->fd, -1, 0, value, NULL);

	count++;
	printf("    %-4d  ", count);

	for (i = 4 - 1; i >= 0; i--)
	{
		bit	= 0x1 << i;

		if (bit & 0x8080)
			printf("  ");

		printf("%s", (value & bit) ? "1" : "0");
	}

	if (count == 1)
	{
		t0		= ts;
		ts_last	= ts;
	}

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * t0.tv_sec + t0.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	if (errs)
		printf("  FAIL <---");

	printf("\n");
	fflush(stdout);

	if (args->ms_wait)
		os_sleep_ms(args->ms_wait);

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs;
	int	i;
	u32	value;

	errs	= _config(args);

	gsc_label("Digital Output");
	printf("\n");
	gsc_label_level_inc();

	_output_header();

	gsc_label_level_inc();

	// Flash
	errs	+= _dev_dio_out(args, 0xF);
	errs	+= _dev_dio_out(args, 0x0);

	// Walking "1"

	for (i = 0; i < 4; i++)
	{
		value	= 1L << i;
		errs	+= _dev_dio_out(args, value);
	}

	// Flash
	errs	+= _dev_dio_out(args, 0x0);
	errs	+= _dev_dio_out(args, 0xF);
	errs	+= _dev_dio_out(args, 0x0);

	// Walking "0"

	for (i = 0; i < 4; i++)
	{
		value	= ~(1L << i) & 0xF;
		errs	+= _dev_dio_out(args, value);
	}

	// Flash
	errs	+= _dev_dio_out(args, 0x0);
	errs	+= _dev_dio_out(args, 0xF);
	errs	+= _dev_dio_out(args, 0x0);

	printf("\n");
	gsc_label("Status");
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	gsc_label_level_dec();
	return(errs);
}



