// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/samples/fsamp/main.c $
// $Rev: 52644 $
// $Date: 2023-03-21 09:52:51 -0500 (Tue, 21 Mar 2023) $

// 24DSI64C200K: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg			values								desc
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",		{ -1 },								"Continue testing until an error occurs."								},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",		{ +1 },								"Continue testing even if errors occur."								},
		{	GSC_ARG_S32_FLAG,	&args->fgen_div,		"-fgena",	{ -1 },								"Set Fgen Div to automatic."											},
		{	GSC_ARG_S32_FLAG,	&args->fgen_div,		"-fgen1",	{ DSI64C200K_FGEN_DIV_1 },			"Set Fgen Div to 1."													},
		{	GSC_ARG_S32_FLAG,	&args->fgen_div,		"-fgen2",	{ DSI64C200K_FGEN_DIV_2 },			"Set Fgen Div to 2."													},
		{	GSC_ARG_S32_FLAG,	&args->fgen_div,		"-fgen4",	{ DSI64C200K_FGEN_DIV_4 },			"Set Fgen Div to 4."													},
		{	GSC_ARG_S32_FLAG,	&args->fgen_div,		"-fgen8",	{ DSI64C200K_FGEN_DIV_8 },			"Set Fgen Div to 8."													},
		{	GSC_ARG_S32_FLAG,	&args->fgen_div,		"-fgen16",	{ DSI64C200K_FGEN_DIV_16 },			"Set Fgen Div to 16."													},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",		{ 1 },								"Continue testing for up to # minutes."									},
		{	GSC_ARG_S32_FLAG,	&args->mclk_div,		"-mclka",	{ -1 },								"Set Mclk Div to automatic."											},
		{	GSC_ARG_S32_FLAG,	&args->mclk_div,		"-mclk4",	{ DSI64C200K_MCLK_DIV_4_FAST },		"Set Mclk Div to 4 (fast)."												},
		{	GSC_ARG_S32_FLAG,	&args->mclk_div,		"-mclk8",	{ DSI64C200K_MCLK_DIV_8_MEDIAN },	"Set Mclk Div to 8 (median)."											},
		{	GSC_ARG_S32_FLAG,	&args->mclk_div,		"-mclk32",	{ DSI64C200K_MCLK_DIV_32_ECO },		"Set Mclk Div to 16 (eco)."												},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",		{ 1 },								"Continue testing for up to # iterations."								},
		{	GSC_ARG_S32_FLAG,	&args->optimal,			"-opt",		{ 1 },								"Limit Nvco and Nref to their optimal range."							},
		{	GSC_ARG_S32_FLAG,	&args->over_sample,		"-osfa",	{ -1 },								"Set Over Sample Factor to automatic."									},
		{	GSC_ARG_S32_FLAG,	&args->over_sample,		"-osf32",	{ DSI64C200K_OVER_SAMPLE_32 },		"Set Over Sample Factor to 32."											},
		{	GSC_ARG_S32_FLAG,	&args->over_sample,		"-osf64",	{ DSI64C200K_OVER_SAMPLE_64 },		"Set Over Sample Factor to 64."											},
		{	GSC_ARG_S32_FLAG,	&args->over_sample,		"-osf128",	{ DSI64C200K_OVER_SAMPLE_128 },		"Set Over Sample Factor to 128."										},
		{	GSC_ARG_S32_FLAG,	&args->over_sample,		"-osf1024",	{ DSI64C200K_OVER_SAMPLE_1024 },	"Set Over Sample Factor to 1024."										},
		{	GSC_ARG_S32_FLAG,	&args->range,			"-r",		{ 1 },								"Compute the rate for all supported sample rates (disables -c and -C)."	},
		{	GSC_ARG_S32_MIN,	&args->fsamp,			"-s",		{ 1 },								"The desired sample rate."												},
		{	GSC_ARG_S32_FLAG,	&args->save,			"-S",		{ 1 },								"Save the rate scan data to fsamp.txt (requires -r)."					},
		{	GSC_ARG_DEV_INDEX,	&args->index,			"index",	{ 1 },								"The zero based index of the device to access."							}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"fsamp",
		/* desc	*/	"Compute the configuration for a desired sample rate.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;

	memset(args, 0, sizeof(args_t));
	args->over_sample	= -1;	// -1 = auto-select
	args->fgen_div		= -1;
	args->fsamp			= 10000;
	args->mclk_div		= -1;	// -1 = auto-select

	errs	= gsc_args_parse(argc, argv, &set);

	if (args->range)
		args->continuous	= 0;

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	gsc_label("Accessing Device");
	printf("%d\n", args->index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)	// A convenience loop.
	{
		gsc_label_init(24);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= dsi64c200k_init_util(1);
		errs	+= os_id_driver(dsi64c200k_open, dsi64c200k_read, dsi64c200k_close);
		errs	+= dsi64c200k_count_boards(1, &args.qty);

		if (args.qty <= 0)
			errs++;

		// Open access to device

		if (errs == 0)
			errs	= dsi64c200k_open_util(args.index, 0, -1, 1, &args.fd);

		if (errs)
			args.continuous	= 0;

		if (errs == 0)
		{
			// Device identification

			errs	= dsi64c200k_id_device(args.fd, -1, 1);

			// Perform testing

			errs	+= perform_tests(&args);

			// Close device access

			errs	+= dsi64c200k_close_util(args.fd, -1, 1);
		}

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((args.qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


