// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/utils/util_channels_ready.c $
// $Rev: 52653 $
// $Date: 2023-03-21 10:02:20 -0500 (Tue, 21 Mar 2023) $

// 24DSI64C200K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi64c200k_channels_ready
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI64C200K_IOCTL_CHANNELS_READY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi64c200k_channels_ready(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Channels Ready", index);

	ret		= dsi64c200k_ioctl(fd, DSI64C200K_IOCTL_CHANNELS_READY, (void*) &set);
	errs	= ret ? 1 : 0;

	if (ret)
	{
		if (errno == ETIMEDOUT)
		{
			set	= DSI64C200K_CHANNELS_READY_NO;
		}
		else
		{
			errs++;

			if (verbose)
				printf("ERROR: dsi64c200k_ioctl() failure, errno = %d\n", errno);
		}
	}

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI64C200K_CHANNELS_READY_NO:

			ptr	= "No, timed out";
			break;

		case DSI64C200K_CHANNELS_READY_YES:

			ptr	= "Yes";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}



//*****************************************************************************
int dsi64c200k_channels_ready_wait(int fd, int index, int verbose, s32* get)
{
	int	errs;
	int	i;
	int	ms		= 0;
	s32	set		= DSI64C200K_CHANNELS_READY_NO;

	if (verbose)
	{
		gsc_label_index("Channels Ready", index);
		printf("waiting ... ");
	}

	for (i = 0; i < 1000; i++)
	{
		errs	= dsi64c200k_channels_ready(fd, -1, 0, 0, &set);

		if ((errs) || (set == DSI64C200K_CHANNELS_READY_YES))
			break;

		os_sleep_ms(1);
		ms	+= 1;
	}

	if (verbose)
	{
		printf(	"%s  (%sReady, %d ms)\n",
				errs ? "FAIL <---" : "PASS",
				(set == DSI64C200K_CHANNELS_READY_YES) ? "" : "Not ",
				ms);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


