// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/utils/util_id.c $
// $Rev: 52653 $
// $Date: 2023-03-21 10:02:20 -0500 (Tue, 21 Mar 2023) $

// 24DSI64C200K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_id_device_pci
*
*	Purpose:
*
*		Identify the device using the PCI registers.
*
*	Arguments:
*
*		fd		The handle used to access the device.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

static int _id_device_pci(int fd)
{
	int	errs;
	u32	reg;

	gsc_label("Vendor ID");
	errs	= dsi64c200k_reg_read(fd, -1, 0, GSC_PCI_9056_VIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Device ID");
	errs	+= dsi64c200k_reg_read(fd, -1, 0, GSC_PCI_9056_DIDR, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x9056)
	{
		printf("(PCI9056)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Sub Vendor ID");
	errs	+= dsi64c200k_reg_read(fd, -1, 0, GSC_PCI_9056_SVID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x10B5)
	{
		printf("(PLX)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	gsc_label("Subsystem ID");
	errs	+= dsi64c200k_reg_read(fd, -1, 0, GSC_PCI_9056_SID, &reg);
	printf("0x%04lX      ", (long) reg);

	if (reg == 0x3575)
	{
		printf("(24DSI64C200K)\n");
	}
	else
	{
		errs++;
		printf("(UNKNOWN) FAIL <---\n");
	}

	return(errs);
}



//*****************************************************************************
static int _id_device_bcfgr(int fd)
{
	const char*	desc;
	int			errs;
	u32			val;

	desc	= dsi64c200k_reg_get_desc(DSI64C200K_GSC_BCFGR);
	gsc_label(desc);
	errs	= dsi64c200k_reg_read(fd, -1, 0, DSI64C200K_GSC_BCFGR, &val);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("0x%08lX\n", (long) val);

	return(errs);
}



/******************************************************************************
*
*	Function:	dsi64c200k_id_device
*
*	Purpose:
*
*		Identify the device.
*
*	Arguments:
*
*		fd		The handle to use to access the device.
*
*		index	The index of the device or -1 if we don't care.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dsi64c200k_id_device(int fd, int index, int verbose)
{
	const char*	desc		= NULL;
	s32			device_type;
	int			errs;

	if (verbose)
		gsc_label_index("Device", index);

	errs	= dsi64c200k_query(fd, -1, 0, DSI64C200K_QUERY_DEVICE_TYPE, &device_type);

	if (errs == 0)
	{
		switch (device_type)
		{
			default:

				errs	= 1;

				if (verbose)
				{
					printf(	"FAIL <---  (unexpected device type: %ld)",
							(long) device_type);
				}

				break;

			case GSC_DEV_TYPE_24DSI64C200K:

				desc	= "24DSI64C200K";
				break;
		}
	}

	if (verbose)
	{
		printf("%s\n", desc ? desc : "");
		gsc_label_level_inc();
	}

	errs	+= _id_device_pci(fd);
	errs	+= _id_device_bcfgr(fd);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}



//*****************************************************************************
int dsi64c200k_id_driver(void)
{
	int	errs;

	errs	= os_id_driver(dsi64c200k_open, dsi64c200k_read, dsi64c200k_close);
	return(errs);
}


