// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/utils/util_irq_sel.c $
// $Rev: 52653 $
// $Date: 2023-03-21 10:02:20 -0500 (Tue, 21 Mar 2023) $

// 24DSI64C200K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi64c200k_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI64C200K_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi64c200k_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ Select", index);

	ret		= dsi64c200k_ioctl(fd, DSI64C200K_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI64C200K_IRQ_INIT_DONE:

			ptr	= "Initialize Done";
			break;

		case DSI64C200K_IRQ_AUTO_CAL_DONE:

			ptr	= "Auto-Calibration Done";
			break;

		case DSI64C200K_IRQ_CHAN_READY_L2H:

			ptr	= "Channels Ready (Low->High)";
			break;

		case DSI64C200K_IRQ_AI_BUF_THR_L2H:

			ptr	= "AI Buffer Threshold (Low->High)";
			break;

		case DSI64C200K_IRQ_AI_BUF_THR_H2L:

			ptr	= "AI Buffer Threshold (high->Low)";
			break;

		case DSI64C200K_IRQ_AI_BURST_DONE:

			ptr	= "Burst Done";
			break;

		case DSI64C200K_IRQ_CHAN_READY_H2L:

			ptr	= "Channels Ready (high->Low)";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


