// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI64C200K/24DSI64C200K_linux_1.x_gsc_dn/utils/util_query.c $
// $Rev: 52116 $
// $Date: 2022-12-01 11:05:52 -0600 (Thu, 01 Dec 2022) $

// 24DSI64C200K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi64c200k_query
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI64C200K_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi64c200k_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128]	= "";
	int			errs		= 0;
	const char*	ptr;
	s32			query		= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case DSI64C200K_QUERY_AUTO_CAL_MS:	ptr	= "Auto-Cal Period";	break;
		case DSI64C200K_QUERY_CHANNEL_MAX:	ptr	= "Max Channels";		break;
		case DSI64C200K_QUERY_CHANNEL_QTY:	ptr	= "Input Channels";		break;
		case DSI64C200K_QUERY_COUNT:		ptr	= "Query Options";		break;
		case DSI64C200K_QUERY_DEVICE_TYPE:	ptr	= "Device Type";		break;
		case DSI64C200K_QUERY_FGEN_MAX:		ptr	= "Fgen Max";			break;
		case DSI64C200K_QUERY_FGEN_MIN:		ptr	= "Fgen Min";			break;
		case DSI64C200K_QUERY_FIFO_SIZE:	ptr	= "FIFO Size";			break;
		case DSI64C200K_QUERY_FILTER_FREQ:	ptr = "Filter Frequency";	break;
		case DSI64C200K_QUERY_FREF_DEFAULT:	ptr	= "Fref Default";		break;
		case DSI64C200K_QUERY_FSAMP_MAX:	ptr	= "Fsamp Max";			break;
		case DSI64C200K_QUERY_FSAMP_MIN:	ptr	= "Fsamp Min";			break;
		case DSI64C200K_QUERY_FW_REV:		ptr	= "Firmware Revision";	break;
		case DSI64C200K_QUERY_INIT_ADC_MS:	ptr	= "ADC Init Period";	break;
		case DSI64C200K_QUERY_INIT_MS:		ptr	= "Initialize Period";	break;
		case DSI64C200K_QUERY_MASTER_CLOCK:	ptr = "Master Clock";		break;
		case DSI64C200K_QUERY_MCLK_MAX:		ptr	= "Mclk Max";			break;
		case DSI64C200K_QUERY_MCLK_MIN:		ptr	= "Mclk Min";			break;
		case DSI64C200K_QUERY_NREF_MAX:		ptr	= "Nref Max";			break;
		case DSI64C200K_QUERY_NREF_MIN:		ptr	= "Nref Min";			break;
		case DSI64C200K_QUERY_NREF_OPT_MAX:	ptr	= "Nref Max Optimal";	break;
		case DSI64C200K_QUERY_NREF_OPT_MIN:	ptr	= "Nref Min Optimal";	break;
		case DSI64C200K_QUERY_NVCO_MAX:		ptr	= "Nvco Max";			break;
		case DSI64C200K_QUERY_NVCO_MIN:		ptr	= "Nvco Min";			break;
		case DSI64C200K_QUERY_NVCO_OPT_MAX:	ptr	= "Nvco Max Optimal";	break;
		case DSI64C200K_QUERY_NVCO_OPT_MIN:	ptr	= "Nvco Min Optimal";	break;
		case DSI64C200K_QUERY_V_RANGE:		ptr	= "Voltage Range";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= dsi64c200k_ioctl(fd, DSI64C200K_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case DSI64C200K_QUERY_CHANNEL_MAX:
		case DSI64C200K_QUERY_CHANNEL_QTY:
		case DSI64C200K_QUERY_COUNT:
		case DSI64C200K_QUERY_NREF_MAX:
		case DSI64C200K_QUERY_NREF_MIN:
		case DSI64C200K_QUERY_NREF_OPT_MAX:
		case DSI64C200K_QUERY_NREF_OPT_MIN:
		case DSI64C200K_QUERY_NVCO_MAX:
		case DSI64C200K_QUERY_NVCO_MIN:
		case DSI64C200K_QUERY_NVCO_OPT_MAX:
		case DSI64C200K_QUERY_NVCO_OPT_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case DSI64C200K_QUERY_FW_REV:

			sprintf(buf, "0x%lX", (long) set);
			break;

		case DSI64C200K_QUERY_FGEN_MAX:
		case DSI64C200K_QUERY_FGEN_MIN:
		case DSI64C200K_QUERY_FILTER_FREQ:
		case DSI64C200K_QUERY_FREF_DEFAULT:
		case DSI64C200K_QUERY_MASTER_CLOCK:
		case DSI64C200K_QUERY_MCLK_MAX:
		case DSI64C200K_QUERY_MCLK_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case DSI64C200K_QUERY_AUTO_CAL_MS:
		case DSI64C200K_QUERY_INIT_ADC_MS:
		case DSI64C200K_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case DSI64C200K_QUERY_FSAMP_MAX:
		case DSI64C200K_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case DSI64C200K_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case DSI64C200K_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_24DSI64C200K)
			{
				strcpy(buf, "24DSI64C200K");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case DSI64C200K_QUERY_V_RANGE:

			if (set == DSI64C200K_QUERY_V_RANGE_10)
			{
				strcpy(buf, "+-10V");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


